==> flutter_smartlook-3.0.6.tar.gz: lib/src/widget_tree/wireframe.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';
import 'dart:ui';

class WidgetNode {
  double? width;
  double? height;
  double? top;
  double? left;
  Color? color;
  bool hasChildrenToVisit = true;
  bool visible = true;

  WidgetNode({
    this.top,
    this.left,
    this.width,
    this.height,
    this.color,
  });

  fillFromElement(Element element) async {
    final screenSize = window.physicalSize / window.devicePixelRatio;
    var renderObject = element.renderObject!;

    if (renderObject is RenderBox && renderObject.hasSize) {
      var offset = renderObject.localToGlobal(Offset.zero);
      top = offset.dy;
      left = offset.dx;

      width = renderObject.size.width;
      height = renderObject.size.height;

      if (top! + screenSize.height < 0) {
        visible = false;
        return;
      }
      if (top! > screenSize.height) {
        visible = false;
        return;
      }
      if (left! + screenSize.width < 0) {
        visible = false;
        return;
      }
      if (left! > screenSize.width) {
        visible = false;
        return;
      }

      if (renderObject is RenderImage && renderObject.image != null) {
        hasChildrenToVisit = false;
        color = Colors.white;
      } else if (renderObject is RenderParagraph) {
        color = renderObject.text.style?.color;
        hasChildrenToVisit = false;
      } else if (renderObject is RenderDecoratedBox &&
          renderObject.decoration is BoxDecoration) {
        BoxDecoration decoration = (renderObject.decoration as BoxDecoration);
        color = decoration.color ?? decoration.gradient?.colors[0];
      } else {
        visible = false;
      }
    } else {
      visible = false;
    }
  }

  factory WidgetNode.fromJson(Map<String, dynamic> json) => WidgetNode(
      top: json["top"],
      left: json["left"],
      width: json["width"],
      height: json["height"],
      color: json["color"]);

  Map<String, dynamic> toJson() => {
        "top": top,
        "left": left,
        "width": width,
        "height": height,
        "color": color?.toHex() ?? Colors.white.toHex(),
      };
}

class WireframeData {
  WireframeData({this.width, this.height, this.items});

  double? width;
  double? height;
  List<WidgetNode>? items;

  fillFromElement(BuildContext buildContext) async {
    var screenSize = window.physicalSize / window.devicePixelRatio;
    width = screenSize.width;
    height = screenSize.height;
    int count = 0;
    Element? rootElement;

    buildContext.visitChildElements((element) {
      count++;
      if (count == 1) {
        rootElement = element;
      }
    });
    if (count > 1) {
      throw FormatException('Only one root element expected!');
    } else {
      items = await getWireframeItemsForElement(rootElement!, 1);
    }
  }

  Future<List<WidgetNode>> getWireframeItemsForElement(
      Element rootElement, int dep) async {
    final elements = collectAllElementsFrom(rootElement, skipOffstage: true);

    List<WidgetNode> items = [];
    for (var element in elements) {
      var node = WidgetNode();
      await node.fillFromElement(element);
      if (node.visible) {
        items.add(node);
      }
    }

    return items;
  }

  Map<String, dynamic> toJson() => {
        "width": width,
        "height": height,
        "items": items!.map((player) => player.toJson()).toList()
      };
}

Iterable<Element> collectAllElementsFrom(
  Element rootElement, {
  required bool skipOffstage,
}) {
  return CachingIterable<Element>(
      _DepthFirstChildIterator(rootElement, skipOffstage));
}

class _DepthFirstChildIterator implements Iterator<Element> {
  _DepthFirstChildIterator(Element rootElement, this.skipOffstage) {
    _fillChildren(rootElement);
  }

  final bool skipOffstage;

  late Element _current;

  final List<Element> _stack = <Element>[];

  @override
  Element get current => _current;

  @override
  bool moveNext() {
    if (_stack.isEmpty) return false;

    _current = _stack.removeLast();
    _fillChildren(_current);

    return true;
  }

  void _fillChildren(Element element) {
    assert(element != null);
    final List<Element> reversed = <Element>[];
    if (skipOffstage) {
      element.debugVisitOnstageChildren(reversed.add);
    } else {
      element.visitChildren(reversed.add);
    }
    while (reversed.isNotEmpty) {
      _stack.add(reversed.removeLast());
    }
  }
}

extension HexColor on Color {
  static Color fromHex(String hexString) {
    final buffer = StringBuffer();
    if (hexString.length == 6 || hexString.length == 7) buffer.write('ff');
    buffer.write(hexString.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }

  String toHex({bool leadingHashSign = true}) => '${leadingHashSign ? '#' : ''}'
      '${alpha.toRadixString(16).padLeft(2, '0')}'
      '${red.toRadixString(16).padLeft(2, '0')}'
      '${green.toRadixString(16).padLeft(2, '0')}'
      '${blue.toRadixString(16).padLeft(2, '0')}';
}

==> flutter_smartlook-4.0.6.tar.gz: lib/src/widget_tree/wireframe.dart <==
import 'dart:async';

import 'dart:ui' as ui;

//import 'package:image/image.dart' as img;
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

class WidgetNode {
  double? width;
  double? height;
  double? top;
  double? left;
  Color? color;
  bool hasChildrenToVisit = true;
  bool visible = true;

  WidgetNode({
    this.top,
    this.left,
    this.width,
    this.height,
    this.color,
  });

  Future<void> fillFromElement(Element element) async {
    final screenSize = ui.window.physicalSize / ui.window.devicePixelRatio;
    final renderObject = element.renderObject!;

    if (renderObject is RenderBox && renderObject.hasSize) {
      final offset = renderObject.localToGlobal(Offset.zero);
      top = offset.dy;
      left = offset.dx;

      width = renderObject.size.width;
      height = renderObject.size.height;

      if (top! + screenSize.height < 0) {
        visible = false;
        return;
      }
      if (top! > screenSize.height) {
        visible = false;
        return;
      }
      if (left! + screenSize.width < 0) {
        visible = false;
        return;
      }
      if (left! > screenSize.width) {
        visible = false;
        return;
      }

      if (renderObject is RenderImage && renderObject.image != null) {
        hasChildrenToVisit = false;
        color = Colors.white;
        //TODO add all images type
        //TODO make better async calls
        // color = await getAverageColorFromImage((element.widget as Image).image);
      } else if (renderObject is RenderParagraph) {
        color = renderObject.text.style?.color;
        hasChildrenToVisit = false;
      } else if (renderObject is RenderDecoratedBox &&
          renderObject.decoration is BoxDecoration) {
        final BoxDecoration decoration =
            renderObject.decoration as BoxDecoration;
        color = decoration.color ?? decoration.gradient?.colors[0];
      } else {
        visible = false;
      }
    } else {
      visible = false;
    }
  }

  //TODO add with wireframe
  /*
  Future<Color?> _getAverageColorFromImage(ImageProvider provider) async {
    final stream = provider.resolve(ImageConfiguration.empty);
    final Completer completer = Completer<ui.Image>();
    final ImageStreamListener listener = ImageStreamListener((frame, sync) {
      final ui.Image image = frame.image;
      completer.complete(image);
    });
    stream.addListener(listener);
    final ui.Image image = await completer.future;
    stream.removeListener(listener);

    final byteData = await image.toByteData();
    final byteList = byteData?.buffer;
    if (byteList == null) return null;

    final img.Image pixelImage =
        img.Image.fromBytes(image.width, image.height, byteList.asUint8List());

    //TODO make better algorithm
    int redBucket = 0;
    int greenBucket = 0;
    int blueBucket = 0;
    int pixelCount = 0;

    final halfWidth = pixelImage.width ~/ 2;
    final tenthHeight = (pixelImage.height / 10).floor();
    for (int i = 0; i < pixelImage.height; i += tenthHeight) {
      final val = pixelImage.getPixel(halfWidth, i);
      pixelCount++;
      redBucket += img.getRed(val);
      greenBucket += img.getGreen(val);
      blueBucket += img.getBlue(val);
    }
    final halfHeight = pixelImage.height ~/ 2;
    final tenthWidth = (pixelImage.width / 10).floor();
    for (int i = 0; i < pixelImage.width; i += tenthWidth) {
      final val = pixelImage.getPixel(i, halfHeight);
      pixelCount++;
      redBucket += img.getRed(val);
      greenBucket += img.getGreen(val);
      blueBucket += img.getBlue(val);
    }
    return Color.fromRGBO(redBucket ~/ pixelCount, greenBucket ~/ pixelCount,
        blueBucket ~/ pixelCount, 1);
  }*/

  factory WidgetNode.fromJson(Map<String, dynamic> json) => WidgetNode(
      top: json["top"],
      left: json["left"],
      width: json["width"],
      height: json["height"],
      color: json["color"]);

  Map<String, dynamic> toJson() => {
        "top": top,
        "left": left,
        "width": width,
        "height": height,
        "color": color?.toHex() ?? Colors.white.toHex(),
      };
}

class WireframeData {
  WireframeData({this.width, this.height, this.items});

  double? width;
  double? height;
  List<WidgetNode>? items;

  Future<void> fillFromElement(BuildContext buildContext) async {
    final screenSize = ui.window.physicalSize / ui.window.devicePixelRatio;
    width = screenSize.width;
    height = screenSize.height;
    int count = 0;
    Element? rootElement;

    buildContext.visitChildElements((element) {
      count++;
      if (count == 1) {
        rootElement = element;
      }
    });
    if (count > 1) {
      throw const FormatException('Only one root element expected!');
    } else {
      items = await getWireframeItemsForElement(rootElement!, 1);
    }
  }

  Future<List<WidgetNode>> getWireframeItemsForElement(
      Element rootElement, int dep) async {
    final elements = collectAllElementsFrom(rootElement, skipOffstage: true);

    final List<WidgetNode> items = [];
    for (final element in elements) {
      final node = WidgetNode();
      await node.fillFromElement(element);
      if (node.visible) {
        items.add(node);
      }
    }

    return items;
  }

  Map<String, dynamic> toJson() => {
        "width": width,
        "height": height,
        "items": items!.map((player) => player.toJson()).toList()
      };
}

Iterable<Element> collectAllElementsFrom(
  Element rootElement, {
  required bool skipOffstage,
}) {
  return CachingIterable<Element>(
      _DepthFirstChildIterator(rootElement, skipOffstage));
}

class _DepthFirstChildIterator implements Iterator<Element> {
  _DepthFirstChildIterator(Element rootElement, this.skipOffstage) {
    _fillChildren(rootElement);
  }

  final bool skipOffstage;

  late Element _current;

  final List<Element> _stack = <Element>[];

  @override
  Element get current => _current;

  @override
  bool moveNext() {
    if (_stack.isEmpty) return false;

    _current = _stack.removeLast();
    _fillChildren(_current);

    return true;
  }

  void _fillChildren(Element element) {
    final List<Element> reversed = <Element>[];
    if (skipOffstage) {
      element.debugVisitOnstageChildren(reversed.add);
    } else {
      element.visitChildren(reversed.add);
    }
    while (reversed.isNotEmpty) {
      _stack.add(reversed.removeLast());
    }
  }
}

extension HexColor on Color {
  static Color fromHex(String hexString) {
    final buffer = StringBuffer();
    if (hexString.length == 6 || hexString.length == 7) buffer.write('ff');
    buffer.write(hexString.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }

  String toHex({bool leadingHashSign = true}) => '${leadingHashSign ? '#' : ''}'
      '${alpha.toRadixString(16).padLeft(2, '0')}'
      '${red.toRadixString(16).padLeft(2, '0')}'
      '${green.toRadixString(16).padLeft(2, '0')}'
      '${blue.toRadixString(16).padLeft(2, '0')}';
}
